﻿using System;
using System.Diagnostics;
using System.Text;
using ASPItalia.Apps4Windows.WinRT.Common;
using Windows.Networking.Connectivity;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;

namespace ASPItalia.Apps4Windows.WinRT.Pages
{
	public sealed partial class BackgroundableBrowser : LayoutAwareUserControl
	{
		public BackgroundableBrowser()
		{
			this.InitializeComponent();
			this.ShowBrowser = true;

			this.Loaded += (s, e) => LayerManager.LayerChanged += LayerManager_LayerChanging;
			this.Unloaded += (s, e) => LayerManager.LayerChanged -= LayerManager_LayerChanging;

			this.webView.LoadCompleted += webView_LoadCompleted;
			this.webView.NavigationFailed += webView_NavigationFailed;
		}

		void webView_LoadCompleted(object sender, Windows.UI.Xaml.Navigation.NavigationEventArgs e)
		{
			// ENG: let's hide the progress ring, the loading is over
			// ITA: nascondiamo il ProgressRing, ha finito di caricare
			this.Progress.Visibility = Visibility.Collapsed;
			this.Progress.IsActive = false;
		}

		void webView_NavigationFailed(object sender, WebViewNavigationFailedEventArgs e)
		{
			// ENG: in case of errors
			// ITA: in caso di errore
			error.Visibility = Visibility.Visible;
		}

		public bool CheckIsNetworkAvailable()
		{
			var connectionProfile = NetworkInformation.GetInternetConnectionProfile();
			return connectionProfile != null &&
				connectionProfile.GetNetworkConnectivityLevel() == NetworkConnectivityLevel.InternetAccess;
		}

		WebViewBrush brush;
		private void LayerManager_LayerChanging(object sender, LayerChangedEventArgs e)
		{
			if (!e.IsTopMostLayer)
			{
				brush = new WebViewBrush();
				brush.SourceName = this.webView.Name;
				brush.Redraw();
				webPlaceholder.Fill = brush;

				if (webView.RenderTransform != null)
				{
					webView.RenderTransform = new TranslateTransform()
					{
						X = -10000
					};
					Debug.WriteLine("webView.RenderTransform.X = -10K;");
				}
			}
			else
			{
				webPlaceholder.Fill = new SolidColorBrush(Colors.Transparent);

				webView.RenderTransform = null;
				brush = null;

				Debug.WriteLine("HidePlaceholder");
			}

			this.ShowBrowser = e.IsTopMostLayer;
		}

		private bool _showBrowser;
		public bool ShowBrowser
		{
			get { return _showBrowser; }
			set
			{
				this.SetProperty(ref _showBrowser, value);
			}
		}

		internal void NavigateToString(string text)
		{
			this.webView.NavigateToString(text);
		}

		internal void Navigate(Uri uri)
		{
			if (uri != null)
			{
				bool hasNetwork = CheckIsNetworkAvailable();

				// ENG: check for network
				// ITA: controlliamo che ci sia rete
				error.Visibility = hasNetwork ? Visibility.Collapsed : Visibility.Visible;

				// ENG: network is ok, let's open the page
				// ITA: c'è rete, navighiamo
				if (hasNetwork)
				{
					// ENG: the progress is displayed to inform the user
					// ITA: attiviamo il progress, così l'utente sa che è in corso di caricamento
					this.Progress.Visibility = Visibility.Visible;
					this.Progress.IsActive = true;

					this.webView.Navigate(uri);
				}
			}
		}

		public static readonly DependencyProperty SourceProperty = DependencyProperty.Register("Source", typeof(Uri), typeof(BackgroundableBrowser), new PropertyMetadata(null, OnSourceChanged));

		private static void OnSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var target = d as BackgroundableBrowser;
			if (target != null)
			{
				target.Navigate((Uri)e.NewValue);
			}
		}

		public Uri Source
		{
			get { return (Uri)GetValue(SourceProperty); }
			set { SetValue(SourceProperty, value); }
		}

		internal void InvokeScript(string scriptName, string[] arguments)
		{
			this.webView.InvokeScript(scriptName, arguments);
			if (brush != null)
			{
				brush.Redraw();
			}
		}
	}
}