﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Common;
using ASPItalia.Apps4Windows.WinRT.Exceptions;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using ASPItalia.Apps4Windows.WinRT.Services;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using ASPItalia.Apps4Windows.WinRT.Resources;

// The Search Contract item template is documented at http://go.microsoft.com/fwlink/?LinkId=234240

namespace ASPItalia.Apps4Windows.WinRT.Pages
{
    /// <summary>
    /// This page displays search results when a global search is directed to this application.
    /// </summary>
    public sealed partial class SearchResultPage : LayoutAwarePage
    {
        private UIElement _previousContent;
        private ApplicationExecutionState _previousExecutionState;

        private bool _canGoBack;
        public bool CanGoBack
        {
            get { return _canGoBack; }
            set { this.SetProperty(ref _canGoBack, value); }
        }

        private string _queryText;
        public string QueryText
        {
            get { return _queryText; }
            set { this.SetProperty(ref _queryText, value); }
        }

        private List<Filter> _filters;
        public List<Filter> Filters
        {
            get { return _filters; }
            set
            {
                this.SetProperty(ref _filters, value);
                this.OnPropertyChanged("ShowFilters");
            }
        }

        public bool ShowFilters
        {
            get { return this.Filters != null && this.Filters.Count > 1; }
        }

        public SearchResultPage()
        {
            this.InitializeComponent();
        }

        private List<SectionItem> _results;
        public List<SectionItem> Results
        {
            get { return _results; }
            set { this.SetProperty(ref _results, value); }
        }

        /// <summary>
        /// Determines how best to support navigation back to the previous application state.
        /// </summary>
        public static void Activate(String queryText, ApplicationExecutionState previousExecutionState)
        {
            var previousContent = Window.Current.Content;
            var frame = previousContent as Frame;

            if (frame == null)
            {
                frame = new Frame();
                Window.Current.Content = frame;
            }

            frame.Navigate(typeof(SearchResultPage), queryText);

            // Either way, active the window
            Window.Current.Activate();
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected override async void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            var queryText = navigationParameter as String;

            var filterList = new List<Filter>();
            var sections = Repository.GetSectionsWithLoaders();

            Task<Filter>[] tasks = sections.Select(x => Filter.BuildAsync(x.Key, x.Value, queryText)).ToArray();

            await Task.WhenAll(tasks);

            filterList = tasks.Where(x => x.IsCompleted)
                .Select(x => x.Result)
                .Where(x => x != null)
                .ToList();

            var allItems = filterList.SelectMany(x => x.Results)
                .OrderByDescending(x => x.PubDate);

            filterList.Insert(0, new Filter(StringDictionary.AllItems, allItems, true));

			// ENG: update the viewmodel
            // ITA: Aggiorno il viewmodel
			this.noResultsTextBlock.Text = StringDictionary.NoResults;
			this.resultText.Text = StringDictionary.Results;
            this.QueryText = '\u201c' + queryText + '\u201d';
            this.CanGoBack = this._previousContent != null;
            this.Filters = filterList;
        }

        /// <summary>
        /// Invoked when the back button is pressed.
        /// </summary>
        /// <param name="sender">The Button instance representing the back button.</param>
        /// <param name="e">Event data describing how the button was clicked.</param>
        protected override void GoBack(object sender, RoutedEventArgs e)
        {
            // Return the application to the state it was in before search results were requested
            if (this.Frame != null && this.Frame.CanGoBack)
            {
                this.Frame.GoBack();
            }
            else if (this._previousContent != null)
            {
                Window.Current.Content = this._previousContent;
            }
            else if (this.Frame != null)
            {
                var app = (App)App.Current;
                app.PerformLaunch(string.Empty, _previousExecutionState);
            }
        }

        /// <summary>
        /// Invoked when a filter is selected using the ComboBox in snapped view state.
        /// </summary>
        /// <param name="sender">The ComboBox instance.</param>
        /// <param name="e">Event data describing how the selected filter was changed.</param>
        void Filter_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Determine what filter was selected
            var selectedFilter = e.AddedItems.FirstOrDefault() as Filter;
            if (selectedFilter != null)
            {
                selectedFilter.Active = true;

                this.Results = selectedFilter.Results;
                if (this.Results.Count > 0)
                {
                    VisualStateManager.GoToState(this, "ResultsFound", true);
                    return;
                }
            }

            // Display informational text when there are no search results.
            VisualStateManager.GoToState(this, "NoResultsFound", true);
        }

        /// <summary>
        /// Invoked when a filter is selected using a RadioButton when not snapped.
        /// </summary>
        /// <param name="sender">The selected RadioButton instance.</param>
        /// <param name="e">Event data describing how the RadioButton was selected.</param>
        void Filter_Checked(object sender, RoutedEventArgs e)
        {
            // Mirror the change into the CollectionViewSource used by the corresponding ComboBox
            // to ensure that the change is reflected when snapped
            if (filtersViewSource.View != null)
            {
                var filter = (sender as FrameworkElement).DataContext;
                filtersViewSource.View.MoveCurrentTo(filter);
            }
        }

        private void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            var item = ((SectionItem)e.ClickedItem);
            this.Frame.NavigateToDetails(item);
        }

        /// <summary>
        /// View model describing one of the filters available for viewing search results.
        /// </summary>
        public sealed class Filter : ASPItalia.Apps4Windows.WinRT.Common.BindableBase
        {
            public static async Task<Filter> BuildAsync(Section section, ILoader loader, string searchKey)
            {
                try
                {
                    var items = await loader.Search(searchKey);

                    return new Filter(section.Title, items.Select(x => new SectionItem(section, x, 0)));
                }
                catch (ContentNotAvailableException)
                {
                    // content not available (offline?), let's return null
                    return null;
                }
            }

            private String _name;
            private bool _active;

            public Filter(String name, IEnumerable<SectionItem> results, bool active = false)
            {
                this.Name = name;
                this.Active = active;

                if (results != null)
                    this.Results = results.ToList();
            }

            public override String ToString()
            {
                return Description;
            }

            public String Name
            {
                get { return _name; }
                set { if (this.SetProperty(ref _name, value)) this.OnPropertyChanged("Description"); }
            }

            public int Count
            {
                get
                {
                    if (this.Results == null)
                        return 0;

                    return this.Results.Count();
                }
            }

            public bool Active
            {
                get { return _active; }
                set { this.SetProperty(ref _active, value); }
            }

            public String Description
            {
                get { return String.Format("{0} ({1})", this.Name, this.Count); }
            }

            private List<SectionItem> _results;
            public List<SectionItem> Results
            {
                get { return _results; }
                set { this.SetProperty(ref _results, value); }
            }
        }
    }
}
