﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Exceptions;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using Windows.Data.Xml.Dom;
using Windows.Networking.Connectivity;
using Windows.Storage;

namespace ASPItalia.Apps4Windows.WinRT.Services
{
    public class RemoteFeedLoader : FeedLoader
    {
        private Uri url;

        public RemoteFeedLoader(Uri url, IconTypeEnum iconType, bool displayImages = true)
        {
            if (url == null)
                throw new ArgumentNullException("url");

            this.url = url;
            this.IconType = iconType;
            this.DisplayImages = displayImages;
        }

        public bool CheckIsNetworkAvailable()
        {
            var connectionProfile = NetworkInformation.GetInternetConnectionProfile();
           
            return connectionProfile != null && 
                connectionProfile.GetNetworkConnectivityLevel() == NetworkConnectivityLevel.InternetAccess;
        }

        public override async Task<IEnumerable<FeedItem>> LoadItemsAsync()
        {
            XmlDocument document = null;

            IStorageFile cacheFile = await Persister.GetFileForUri(url, createIfNotFound: false);

            if (!this.CheckIsNetworkAvailable())
            {
				// ENG: no network. Do we have a recent file?
                // ITA: rete non disponibile, c'è un file recente?
                if (cacheFile == null)
                    throw new ContentNotAvailableException();

                if ((DateTime.Now - cacheFile.DateCreated) > TimeSpan.FromDays(3))
                {
					// ENG: file too old, let's delete it
                    // ITA: il file è troppo vecchio, lo eliminiamo
#pragma warning disable 4014
                    cacheFile.DeleteAsync();
#pragma warning restore 4014
                    throw new ContentNotAvailableException();
                }

				// ENG: the first is recent, let's get it
                // ITA: il file è abbastanza recente, lo restituiamo
                document = await XmlDocument.LoadFromFileAsync(cacheFile);
            }
            else
            {
				// ENG: get file info
                // ITA: Recupero le informazioni sul file
                var properties = (cacheFile != null) ? await cacheFile.GetBasicPropertiesAsync() : null;

				// ENG: network ready, file is recent, so we're going to use it
                // ITA: rete disponibile, il file è molto recente e lo riutilizzo
                if (cacheFile != null && (DateTime.Now - properties.DateModified) < TimeSpan.FromMinutes(5))
                {
                    document = await XmlDocument.LoadFromFileAsync(cacheFile);
                }
                else
                {
                    if (cacheFile != null)
                    {
                        await cacheFile.DeleteAsync();
                    }

					// ENG: get the file from the network and save its content for future requests
                    // ITA: recupero dalla rete e salvo il contenuto sul file per il prossimo caricamento
                    document = await XmlDocument.LoadFromUriAsync(url);

                    cacheFile = await Persister.GetFileForUri(url, createIfNotFound: true);
#pragma warning disable 4014
                    document.SaveToFileAsync(cacheFile);
#pragma warning restore 4014
                }
            }
            
			// ENG: parse the XML file and get the FeddItems
            // ITA: parsing dell'XML e ottengo i FeedItem
            return LoadFromXML(document);
        }

    }
}
