using System;
using System.Diagnostics;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Common;
using ASPItalia.Apps4Windows.WinRT.Model;
using ASPItalia.Apps4Windows.WinRT.Services;
using Windows.ApplicationModel;
using Windows.Storage;
using Windows.UI.Core;
using Windows.UI.Xaml;
using System.Linq;
using Windows.UI.Xaml.Media;
using Windows.UI.ViewManagement;
using System.Net;
using ASPItalia.Apps4Windows.WinRT.Extensions;

namespace ASPItalia.Apps4Windows.WinRT.Pages
{
	public sealed partial class ContentDisplay : LayoutAwareUserControl
	{
		private bool textSizeInvalidated;

		public ContentDisplay()
		{
			this.InitializeComponent();

			// ENG: to prevent the WebUI control from flickering, we hide the elements,
			// and display them after 2 seconds
			// ITA: rendiamo subito invisibile, poi lo rendiamo visibile dopo 2 secondi
			// evita il problema di flickering del controllo WebUI
			ShowWebUI();

			// ENG: loading handler
			// ITA: sottoscriviamo eventi al caricamento
			this.Loaded += (s, e) =>
			{
				// ENG: text size
				// ITA: gestione del cambio di dimensione testo
				ServiceLocator.Settings.TextSizeChanged += Settings_TextSizeChanged;

				// ENG: windows's size changed
				// ITA: cambio di dimensione della finestra
				Window.Current.SizeChanged += Current_SizeChanged;
			};


			// END: unsubscribe from events while unloading
			// ITA: desottoscriviamo eventi all'uscita
			this.Unloaded += (s, e) =>
			{
				// ENG: text size
				// ITA: gestione del cambio di dimensione testo
				ServiceLocator.Settings.TextSizeChanged -= Settings_TextSizeChanged;

				// ENG: windows's size changed
				// ITA: cambio di dimensione della finestra
				Window.Current.SizeChanged -= Current_SizeChanged;

				// ENG: if the web page has flash content, this call ensures that the page
				// doesn't stay active after we move to another view
				// ITA: Serve a gestire la casistica di una pagina web con contenuto flash,
				// che rimane attivo anche dopo essere navigati via dalla vista corrente
				this.webContent.NavigateToString(string.Empty);
			};
		}

		private void ShowWebUI()
		{
			this.Container.Visibility = Visibility.Collapsed;
			this.Progress.Visibility = Visibility.Visible;

			var timer = new DispatcherTimer();
			timer.Interval = new TimeSpan(0, 0, 2);
			timer.Tick += (e, o) =>
			{
				this.Container.Visibility = Visibility.Visible;
				this.Progress.Visibility = Visibility.Collapsed;
				timer.Stop();
			};
			timer.Start();
		}

		ApplicationViewOrientation lastOrientation = ApplicationViewOrientation.Landscape;
		bool wasSnapped = false;

		void Current_SizeChanged(object sender, WindowSizeChangedEventArgs e)
		{
			bool orientationChanged = lastOrientation != ApplicationView.GetForCurrentView().Orientation;

			// gestiamo il cambio di layout solo se cambia l'orientamento
			if (orientationChanged)
			{
				this.Dispatcher.RunAsync(CoreDispatcherPriority.Low, () =>
				{
					ShowWebUI();
					ShowTextContentAync();
				});
			}

			lastOrientation = ApplicationView.GetForCurrentView().Orientation;
		}

		public SectionItem Item
		{
			get { return (SectionItem)GetValue(ItemProperty); }
			set { SetValue(ItemProperty, value); }
		}

		public Style HeaderStyle
		{
			get { return (Style)GetValue(HeaderStyleProperty); }
			set { SetValue(HeaderStyleProperty, value); }
		}

		// Using a DependencyProperty as the backing store for HeaderStyle.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty HeaderStyleProperty =
			DependencyProperty.Register("HeaderStyle", typeof(Style), typeof(ContentDisplay), new PropertyMetadata(null, HeaderStyleCallBack));


		public Style SubTitleStyle
		{
			get { return (Style)GetValue(SubTitleStyleProperty); }
			set { SetValue(SubTitleStyleProperty, value); }
		}

		// Using a DependencyProperty as the backing store for SubTitleStyle.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty SubTitleStyleProperty =
			DependencyProperty.Register("SubTitleStyle", typeof(Style), typeof(ContentDisplay), new PropertyMetadata(null, SubTitleStyleCallaBack));

		public static void SubTitleStyleCallaBack(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			ContentDisplay control = (ContentDisplay)d;
			control.richTextBlock.Style = (Style)e.NewValue;
		}

		public static void HeaderStyleCallBack(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			ContentDisplay control = (ContentDisplay)d;
			control.pageTitle.Style = (Style)e.NewValue;
		}

		public static readonly DependencyProperty ItemProperty =
			DependencyProperty.Register("Item", typeof(SectionItem), typeof(ContentDisplay), new PropertyMetadata(null, HandleItemChanged));

		private static void HandleItemChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			ContentDisplay control = (ContentDisplay)d;
			SectionItem newValue = e.NewValue as SectionItem;

			if (newValue != null && newValue.IsWebContent)
			{
				if (!string.IsNullOrWhiteSpace(newValue.Content))
				{
					control.ShowTextContentAync();
				}
				else
				{
					control.webContent.Navigate(newValue.Uri);
				}
			}
		}

		// ENG: base HTML and CSS, including the JavaScript to change font size.
		// ITA: l'HTML di base, in cui carichiamo anche il CSS
		// nell'HTML c' il JavaScript per cambiare dimensione del testo
		private const string HTMLTEMPLATE = "<!DOCTYPE html>\r\n<html>\r\n<head><script type=\"text/javascript\">\r\nwindow.onerror = function() {{ return true; }};\r\nfunction changeSize(s) {{ document.getElementById('container').style.fontSize = s + \"px\";}}</script><style type=\"text/css\">{0}</style></head><body class=\"{3}\"><div id='container'>{2}{1}</div></div></body></html>";
		private async void ShowTextContentAync()
		{
			// ENG: mode is changing
			// ITA: segnalo il cambio di modalit
			OnPropertyChanged("IsImmersive");

			// ENG: get the formatted HTML
			// ITA: formattiamo l'HTML da mostrare
			var item = this.Item;

			this.ApplyImmersive(this.IsImmersive);

			if (item == null)
				return;

			string html = null;
			if (item.IsVideoGallery)
			{
				Uri uri = item.Video ?? item.Uri;
				string host = uri.Host.ToLower();

				// ENG: special case for youtube
				// ITA: Caso speciale per youtube
				if (host == "www.youtube.com" || host == "youtu.be")
				{
					string id = null;
					// ENG: share format http://youtu.be/i1GNDs7DCTw
					// ITA: Formato di share http://youtu.be/i1GNDs7DCTw
					if (host == "youtu.be")
						id = uri.AbsolutePath.Substring(1);
					else
					{
						var q = uri.ParseQueryString();
						q.TryGetValue("v", out id);
					}
					string u = "http://www.youtube.com/embed/" + id;
					html = String.Format("<html style=\"width: 100%; height: 100%\"><body style=\"width: 100%; height: 100%; margin: 0px; padding: 0px; overflow: hidden\"><iframe style=\"width: 100%; height: 100%; overflow: hidden\" scrolling=\"no\" src=\"{0}\" frameborder=\"0\" allowfullscreen></iframe></body></html>", u);
				}
				// ENG: let's try with the video URL
				// ITA: Provo direttamente con l'URI del video
				else if (item.Video != null)
				{
					html = String.Format("<html style=\"width: 100%; height: 100%\"><body style=\"width: 100%; height: 100%; margin: 0px; padding: 0px; overflow: hidden\"><video style=\"width: 100%; height: 100%; overflow: hidden\" controls><source src=\"{0}\"></video></body></html>", item.Video);
				}
			}

			// ENG: it's not a video, so we can show the HTML
			// ITA: non  un video oppure non ho potuto trovare un modo per visualizzarlo, mostriamo l'HTML
			if (String.IsNullOrEmpty(html))
			{
				// ENG: the column layout is used if we are in landscape and the section type is news
				// ITA: mostriamo il layout a colonne solo se siamo in landscape e la sezione  di tipo news
				var containerDiv = IsImmersive
					? string.Format("<div class='columns'><div class='headergroup'><div class=\"header\">{0}</div><div class=\"subheader\">{1}</div></div>", item.Section.Title, item.Title) : "<div>";

				// ENG: the CSS is located in Assets\template.css - you can modify it to alter the visual results
				// ITA: viene applicato il CSS definito in Assets\template.css - pu essere modificato per cambiare aspetto visuale
				var style = await this.GetStyle();
				html = string.Format(HTMLTEMPLATE, style, item.Content, containerDiv, IsImmersive ? "immersive" : ApplicationView.Value.ToString().ToLower());
			}

			await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
			{
				this.webContent.NavigateToString(html);
				this.webContent.Focus(Windows.UI.Xaml.FocusState.Programmatic);
			});
		}

		// ENG: the CSS is loaded and customized using the XML templates
		// ITA: carichiamo il CSS in maniera tale che sia customizzabile con gli stessi colori del template XAML
		private async Task<string> GetStyle()
		{
			// ENG: let's load the file
			// ITA: caricamento del file
			StorageFolder folder = await Package.Current.InstalledLocation.GetFolderAsync("Assets");
			StorageFile file = await folder.GetFileAsync("template.css");

			string css = await FileIO.ReadTextAsync(file);

			// ENG: the resource are loaded from the dictionary
			// ITA: cerco le risorse nei dizionari
			var resources = App.Current.Resources.MergedDictionaries.First(x => x.Source == new Uri("ms-resource:/Files/Common/StandardStyles.xaml")).MergedDictionaries.First(x => x.Source == new Uri("ms-resource:/Files/Assets/CustomStyles.xaml"));

			// ENG: get the CSS using the options
			// ITA: Applico la dimensione scelta nelle opzioni
			css = String.Format(css, ServiceLocator.Settings.TextSize,
				ExtractHexColor((SolidColorBrush)resources["ListViewItemOverlayForegroundBrush"]),
				ExtractHexColor((SolidColorBrush)resources["ApplicationPageBackgroundBrush"]),
				ExtractHexColor((SolidColorBrush)resources["ListViewItemSelectedBackgroundBrush"]),
				this.ActualHeight - 30, // ENG: container height - ITA altezza del contenitore
				this.ActualHeight - 40 // ENG: column hight in immersive view - ITA: altezza delle colonne nelle viste immersive
				);

			return css;
		}

		private string ExtractHexColor(SolidColorBrush value)
		{
			return "#" + value.Color.ToString().Substring(3);
		}

		private void LayerManager_LayerChanged(object sender, LayerChangedEventArgs e)
		{
			if (textSizeInvalidated)
				ShowTextContentAync();
			textSizeInvalidated = false;
		}

		private void Settings_TextSizeChanged(object sender, EventArgs e)
		{
			textSizeInvalidated = true;
			webContent.InvokeScript("changeSize", new[] { ServiceLocator.Settings.TextSize.ToString() });
		}

		/// <summary>
		/// ENG: Returns true if the view is immersive
		/// ITA: Indica se la view  immersive
		/// </summary>
		public bool IsImmersive
		{
			get
			{
				return this.Item != null && this.Item.Section.Type == SectionTypeEnum.News && (ApplicationView.Value == ApplicationViewState.Filled || ApplicationView.Value == ApplicationViewState.FullScreenLandscape);
			}
		}

		private void ApplyImmersive(bool immersive)
		{
			// ENG: change the margin for immersive view
			// ITA: gestisco il margine per le view immersive
			if (ApplicationView.Value != ApplicationViewState.FullScreenPortrait && ApplicationView.Value != ApplicationViewState.Snapped)
			{
				this.Margin = immersive ? new Thickness(-5) : this.Margin;
				if (immersive)
				{
					this.Width = Window.Current.Bounds.Width;
					this.Height = Window.Current.Bounds.Height;
				}
			}
			//else
			//  this.Margin = new Thickness();

			if (immersive)
			{
				TitleHeader.Height = new GridLength(0);
			}
			else
			{
				//TitleHeader.Height = new GridLength(120);
			}
			this.ShowHeader = !immersive;
		}

		private bool _showHeader = true;
		/// <summary>
		/// ENG: returns true if the header must be shown with the content.
		/// ITA: Indica se l'header va visualizzata insieme al contenuto
		/// </summary>
		public bool ShowHeader
		{
			get
			{
				return _showHeader;
			}
			set
			{
				if (_showHeader != value)
				{
					_showHeader = value;
					OnPropertyChanged();
				}
			}
		}
	}
}